      SUBROUTINE O2_ABS(TEMP,PRES,VAPDEN,NFREQ,FREQ,ABO2)
C     RETURNS POWER ABSORPTION COEFFICIENT DUE TO OXYGEN IN AIR,
C     IN NEPERS/KM.  MULTIPLY ABO2 BY 4.343 TO CONVERT TO DB/KM.
C
C      5/1/95  P. Rosenkranz 
C      11/5/97  P. Rosenkranz - 1- line modification.
c      12/16/98 pwr - updated submm freq's and intensities from HITRAN96
c      8/21/02  pwr - revised width at 425
c      3/20/03  pwr - 1- line mixing and width revised
c      9/29/04  pwr - new widths and mixing, using HITRAN intensities
c                     for all lines
c      6/12/06  pwr - chg. T dependence of 1- line to 0.8
C      10/14/08 pwr - moved isotope abundance back into intensities, 
c                     added selected O16O18 lines.
c      5/30/09  pwr - remove common block, add weak lines.
c      12/18/14 pwr - adjust line broadening due to water vapor.
C      9/29/18  pwr - 2nd-order line mixing
c      12/12/18 speed-dependent parameters only for 1- line.
c      Aug. 25, 2022 - submm intensities updated.
c      Feb. 28, 2023 - revised 1st-order coeff. and intensities
c      Mar. 29, 2023 - renamed as subroutine, FREQ now a vector
c      July 24, 2023 - adjustment of mixing
c      Oct. 9, 2023  - G orthogonal to A
C      June 17, 2024 - bias adjustment of y
C
      IMPLICIT NONE
C
C     ARGUMENTS:
C          NAME          UNITS    I/O  DESCRIPTON            VALID RANGE
      REAL  TEMP       ! KELVIN    I   TEMPERATURE  uncertain, but probably
c                                                   valid for atmosphere
      REAL  PRES       ! MILLIBAR  I   PRESSURE              3 TO 1000
      REAL VAPDEN      ! G/M**3    I   WATER VAPOR DENSITY (ENTERS LINEWIDTH
C                  CALCULATION DUE TO GREATER BROADENING EFFICIENCY OF H2O)
      INTEGER NFREQ    !           I   number of frequencies
      REAL FREQ(NFREQ) !  GHZ      I   FREQUENCY              0 to 900
      REAL ABO2(NFREQ) ! NEPERS/KM O   POWER ABSORPTION COEFFICIENT
C
C REFERENCES:
C [1] P.W. Rosenkranz, Chapter 2 in ATMOSPHERIC REMOTE SENSING
C     BY MICROWAVE RADIOMETRY (M.A. Janssen, ed., 1993) 
C    (http://hdl.handle.net/1721.1/68611).
c [2] A.S. Pine, JQSRT v.57(2), pp.145-155 (1997).
c [3] JPL Molecular Spectroscopy, https://spec.jpl.nasa.gov/
c [4] Golubiatnikov & Krupnov, J. Mol. Spect. v.217, pp.282-287 (2003)
c [5] B.J. Drouin, JQSRT v.105, pp.450-458 (2007).
C [6] Tretyakov et al, J. Mol. Spect. v.231, pp.1-14 (2005).
c [7] Koshelev et al, JQSRT, v.169, pp.91-95 (2016).
c [8] Koshelev et al, JQSRT v.154, pp.24-27 (2015).
c [9] Makarov et al, JQSRT v.243 Mar. 2020, doi:10.1016/j.jqsrt.2019.106798
c [10] Rosenkranz and Cimini, IEEE-TGRS v.57 (12), pp.9702-9708 (2019).
c [11] Tomiyama et al, App. Optics v.26 (10), pp. 2020-2028 (1987).
c [12] Bulanin et al, JQSRT v.31,  pp.521-543 (1984).
c [13] Koshelev et al, JQSRT v.196, pp.78-86 (2017).
c [14] Makarov et al, J. Mol. Spect. v.252, pp.242-243 (2008).
c
c Notes: The same temperature dependence (X) is used for submillimeter 
c   line widths as in the 60 GHz band: (300/T)**X 
c   The 1- line includes speed-dependence but only first-order mixing.
c   First-order line shift is set to zero, based on lab measurements. 
c   Line mixing for submm lines is not considered.
C
c     Local variables:
      INTEGER J,K,NL
      PARAMETER (NL=49)
      REAL TH,TH1,PRESWV,PRESDA,WNR,Y(NL),SF1,SF2,FCEN,FJ,YK,ADJY
      REAL WIDTH2,X,WB300,W300(NL),F(NL),Y300(NL),S300(NL),Y1(NL),BE(NL)
      REAL G0(NL),G1(NL),PE1,PE2,A(NL),G(NL),WIDTH,GFAC,RATIO,SUMY2
      REAL DNU0(NL),DNU1(NL)
      DOUBLE PRECISION ASQ,SUMM,SUMY,SUMG,ANORM
      double complex dcerror,Xc,Xrt,pxw,ASD

c  LINES ARE ARRANGED 1-,1+,...37-,37+ IN SPIN-ROTATION SPECTRUM;
c  BY FREQUENCY IN SUBMM SPECTRUM.
C     F,S300,BE from [3]
      DATA F/118.7503, 56.2648, 62.4863, 58.4466, 60.3061, 59.5910,
     & 59.1642, 60.4348, 58.3239, 61.1506, 57.6125, 61.8002,
     & 56.9682, 62.4112, 56.3634, 62.9980, 55.7838, 63.5685,
     & 55.2214, 64.1278, 54.6712, 64.6789, 54.1300, 65.2241,
     & 53.5958, 65.7648, 53.0669, 66.3021, 52.5424, 66.8368,
     & 52.0214, 67.3696, 51.5033, 67.9008, 50.9877, 68.4310,
     & 50.4742, 68.9603, 
     & 233.9461, 368.4982, 401.7398, 424.7630,
     & 487.2493, 566.8956, 715.3929, 731.1867,
     & 773.8395, 834.1456, 895.0711/
      DATA S300/ 0.2925E-14, 0.8011E-15, 0.2460E-14, 0.2208E-14,
     & 0.3323E-14, 0.3265E-14, 0.3690E-14, 0.3859E-14,
     & 0.3610E-14, 0.3974E-14, 0.3200E-14, 0.3686E-14,
     & 0.2607E-14, 0.3131E-14, 0.1967E-14, 0.2459E-14,
     & 0.1382E-14, 0.1796E-14, 0.9074E-15, 0.1225E-14,
     & 0.5581E-15, 0.7818E-15, 0.3222E-15, 0.4684E-15,
     & 0.1749E-15, 0.2636E-15, 0.8937E-16, 0.1397E-15,
     & 0.4301E-16, 0.6968E-16, 0.1952E-16, 0.3277E-16,
     & 0.8354E-17, 0.1454E-16, 0.3377E-17, 0.6091E-17,
     & 0.1289E-17, 0.2409E-17,
     & 0.8298E-17, 0.6506E-15, 0.1347E-16, 0.7097E-14,
     & 0.3032E-14, 0.1815E-16, 0.1839E-14, 0.2215E-16,
     & 0.1160E-13, 0.3999E-14, 0.2537E-16/
      DATA BE/  0.0095,  0.0145,  0.0829,  0.0833,  0.2072,  0.2073,
     &  0.3865,  0.3864,  0.6209,  0.6207,  0.9103,  0.9100,
     &  1.2548,  1.2543,  1.6542,  1.6536,  2.1084,  2.1078,
     &  2.6175,  2.6168,  3.1814,  3.1806,  3.7998,  3.7989,
     &  4.4729,  4.4719,  5.2003,  5.1992,  5.9821,  5.9809,
     &  6.8180,  6.8168,  7.7080,  7.7067,  8.6520,  8.6506,
     &  9.6496,  9.6481,
     &  0.0187, 0.0485, 0.0448, 0.0440,
     &  0.0490, 0.0838, 0.1452, 0.1359,  0.1405, 0.1453, 0.2010/

C  WIDTHS IN GHz/bar from [4,5,6,7,13,14] (1- width = gamma_0)
      DATA WB300/.56/, X/.754/
      DATA W300/1.711, 1.703, 1.513, 1.495, 1.433, 1.408,
     & 1.353, 1.353, 1.303, 1.319, 1.262, 1.265, 
     & 1.238, 1.217, 1.207, 1.207, 1.137, 1.137,
     & 1.101, 1.101, 1.037, 1.038, 0.996, 0.996,
     & 2*0.955, 2*0.906, 2*0.858, 
     & 2*0.811, 2*0.764, 2*0.717, 
     & 2*0.669, 1.65, 3*1.64, 4*1.60, 1.62, 2*1.47/
C
c 1st-order mixing coeff. in 1/bar from [9]
      DATA Y300/ -0.041, 0.277, -0.372, 0.559, -0.573, 0.618,
     & -0.366, 0.278, -0.089, -0.021, 0.060, -0.152,
     & 0.216, -0.293, 0.373, -0.436, 0.491, -0.542,
     & 0.571, -0.613, 0.636, -0.670, 0.690, -0.718,
     & 0.740, -0.763, 0.788, -0.807, 0.834, -0.849,
     & 0.876, -0.887, 0.915, -0.922, 0.950, -0.955,
     & 0.987, -0.988, 11*0./
      DATA Y1/ 0., 0.124, -0.002, 0.008, 0.045, -0.093,
     & 0.264, -0.351, 0.359, -0.416, 0.326, -0.353,
     & 0.484, -0.503, 0.579, -0.590, 0.616, -0.619,
     & 0.611, -0.609, 0.574, -0.568, 0.574, -0.566,
     & 0.60, -0.59, 0.63, -0.62, 0.64, -0.63, 0.65,
     & -0.64, 0.65, -0.64, 0.65, -0.64, 0.64, -0.62, 11*0./
c
c 2nd-order mixing coeff. in 1/bar^2 from [9] (except 1-)
      DATA G0/ 0., -0.090, -0.103, -0.239, -0.172, -0.171,
     & 0.028, 0.150, 0.132, 0.170, 0.087, 0.069, 
     & 0.083, 0.067, 0.007, 0.016, -0.021, -0.066,
     & -0.095, -0.115, -0.118, -0.140, -0.173, -0.186,
     & -0.217, -0.227, -0.234, -0.242, -0.266, -0.272,
     & -0.301, -0.304, -0.334, -0.333, -0.361, -0.358,
     & -0.348, -0.344, 11*0./
      DATA G1/ 0., -0.045, 0.007, 0.033, 0.081, 0.162,
     & 0.179, 0.225, 0.054, 0.003, 0.0004, -0.047,
     & -0.034, -0.071, -0.180, -0.210, -0.285, -0.323,
     & -0.363, -0.380, -0.378, -0.387, -0.392, -0.394,
     & -0.424, -0.422, -0.465, -0.46, -0.51, -0.50,
     & -0.55, -0.54, -0.58, -0.56, -0.62, -0.59, -0.68, -0.65, 11*0./
c dnu in GHz/bar^2
      DATA DNU0/ -0.00028, 0.00597, -0.0195, 0.032, -0.0475, 0.0541,
     & -0.0232, 0.0154, 0.0007, -0.0084, -0.0025, -0.0014,
     & -0.0004, -0.002, 0.005, -0.0066, 0.0072, -0.008,
     & 0.0064, -0.007, 0.0056, -0.006, 0.0047, -0.0049,
     & 0.0040, -0.004, 0.0036, -0.0037, 0.0033, -0.0034,
     & 0.0032, -0.0032, 0.0030, -0.0030, 0.0028, -0.0029,
     & 0.0029, -0.0029, 11*0./
      DATA DNU1/ -0.00039, 0.009, -0.012, 0.016, -0.027, 0.029,
     & 0.006, -0.015, 0.010, -0.014, -0.013, 0.013,
     & 0.004, -0.005, 0.010, -0.010, 0.010, -0.011,
     & 0.008, -0.009, 0.003, -0.003, 0.0009, -0.0009,
     & 0.0017, -0.0016, 0.0024, -0.0023, 0.0024, -0.0024,
     & 0.0024, -0.0020, 0.0017, -0.0016, 0.0013, -0.0012,
     & 0.0005, -0.0004, 11*0./
C
c  Line parameters dependent only on TEMP, PRES, and VAPDEN
      TH = 300./TEMP
      TH1 = TH-1.
      PRESWV = 4.615228E-3*VAPDEN*TEMP
      PRESDA = PRES -PRESWV
      PE1 = .001*(PRESDA*TH**X + 1.2*PRESWV*TH) ![8] includes common TEMP-dependence
      ADJY = .99 ! adjustment following update of line intensities
      PE2 = PE1**2
      WNR = WB300*PE1
      ANORM = 0.D0
      SUMY = 1.584E-17*WB300
      SUMG = 0.D0
      ASQ = 0.D0
      DO K=1,NL
       A(K) = S300(K)*EXP(-BE(K)*TH1)*TH/F(K)**2
       Y(K) = ADJY*(Y300(K)+Y1(K)*TH1)
c With increasing frequency, the spin-rotation spectrum (including zero-
c and negative-freq lines) should approach an asymptote(SUMY) = zero [6,12].
       IF(K.LE.38) SUMY = SUMY + 2.*A(K)*(W300(K)+Y(K)*F(K))
       G(K) = G0(K) + G1(K)*TH1
c Eq.6 in [2] requires G to be orthogonal to A (SUMG=0);
c then the integrated spectral density [11] will be the same as
c with a VanVleck-Weisskopf shape factor.
       IF(K.GT.1 .AND. K.LE.38) THEN
         SUMG = SUMG + A(K)*G(K)
         ASQ = ASQ + DBLE(A(K))**2
         ANORM = ANORM + A(K) ! The bias adjustment for Y is applied
c from K=2 to 38; therefore it is normalized by ANORM summed from 2 to 38.
       ENDIF
      END DO
      SUMY2 = SUMY/(2.D0*ANORM)
      RATIO = SUMG/ASQ
      DO K=2,38
c Y(1) is not adjusted because it was measured separately.
        Y(K) = Y(K) - SUMY2/F(K) !bias adjustment
        G(K) = G(K) - A(K)*RATIO ! makes G orthogonal to A
      END DO
c
      DO 40 J=1,NFREQ
      FJ = FREQ(J)
c  1.571e-17 (o16-o16) + 1.3e-19 (o16-o18) = 1.584e-17
      SUMM = 1.584E-17*WNR/(FJ*FJ + WNR*WNR) ! zero-frequency term
      DO 32 K=1,NL
      WIDTH = W300(K)*PE1
      YK = PE1*Y(K)
      IF(K.GT.1 .AND. K.LE.38) THEN
       GFAC = 1. + PE2*G(K)
      ELSE
       GFAC = 1.
      ENDIF
      FCEN = F(K) + PE2*(DNU0(K)+DNU1(K)*TH1)
c-------------------speed-dependence section; see [10]
      IF(K.EQ.1 .AND. ABS(FJ-FCEN).LT.10.*WIDTH) THEN
C       speed-dependent resonant shape factor with no 1st-order shift
        WIDTH2 = .076*WIDTH ! from [13]
        Xc = cmplx((WIDTH-1.5*WIDTH2),(FJ-FCEN))/WIDTH2
        Xrt = cdsqrt(Xc)
        pxw = 1.77245385090551603d0*Xrt*dcerror(-dimag(Xrt),dble(Xrt))
        ASD = cmplx(1.,YK)*2.d0*(1.d0-pxw)/WIDTH2
        SF1 = REAL(ASD)
      ELSE
c-------------------
      SF1 = (WIDTH*GFAC + (FJ-FCEN)*YK)/((FJ-FCEN)**2 + WIDTH**2)
      ENDIF
      SF2 = (WIDTH*GFAC - (FJ+FCEN)*YK)/((FJ+FCEN)**2 + WIDTH**2)
      SUMM = SUMM + A(K)*(SF1+SF2)
32    CONTINUE !end of line loop (K)
c   .20946e-4/(3.14159*1.38065e-19*300) = 1.6097e11
      ABO2(J) = 1.6097E11*SUMM*PRESDA*(FJ*TH)**2
40    CONTINUE !end of freq loop (J)
      RETURN
      END
