      SUBROUTINE O2ABSZ(TEMP,PRESDA,FREQ,BFIELD,LINE,ALPHA,GM,GPI,GP)
C
C  RETURNS AMPLITUDE PROPAGATION COEFFICIENTS FOR ONE ZEEMAN-SPLIT
C  OXYGEN LINE IN DRY AIR, SPECIFIED BY LINE INDEX, AND POWER ABSORPTION
C  COEFFICIENT DUE TO OTHER OXYGEN LINES.
C
C      1/14/2019  P. Rosenkranz - based on 12/18/2014 version of O2ABS
c      Aug. 25,2022 - submm intensities updated
c      July 24, 2023 - line parameters consistent with o2_abs.f
C
      IMPLICIT NONE
C
C     ARGUMENTS:
      INTEGER LINE
      REAL TEMP,PRESDA,FREQ,BFIELD,ALPHA
      COMPLEX GM,GPI,GP
C
C  NAME    IN/OUT  UNITS    DESCRIPTION        VALID RANGE
C  TEMP      I     KELVIN   TEMPERATURE        UNCERTAIN, but believed to be
c                                          valid for atmosphere
C  PRESDA    I    MILLIBAR PRESSURE           3 TO 1000
C  FREQ      I    GHZ      FREQUENCY          0 TO 900
C  BFIELD    I    GAUSS    MAGNETIC FIELD MAGNITUDE
C  LINE      I             INDEX NUMBER OF LINE IN F ARRAY
C  ALPHA     O    1/KM     POWER ABSORPTION COEFF FOR OTHER LINES
C  GM        O    1/KM     AMPLITUDE PROPAGATION COEFF FOR DELTA M = -1
C  GPI       O    1/KM     AMPLITUDE PROPAGATION COEFF FOR DELTA M =  0
C  GP        O    1/KM     AMPLITUDE PROPAGATION COEFF FOR DELTA M = +1
C
C     REFERENCES FOR EQUATIONS AND COEFFICIENTS:
C     P.W. Rosenkranz, CHAP. 2 in ATMOSPHERIC REMOTE SENSING
C       BY MICROWAVE RADIOMETRY (M.A. Janssen, ed., 1993) 
C       (http://hdl.handle.net/1721.1/68611).
c     G.Yu. Golubiatnikov & A.F. Krupnov, J. Mol. Spect. v.217, 
c       pp.282-287 (2003).
C     M.Yu. Tretyakov et al, J. Mol. Spect. v.223, pp.31-38 (2004).
C     M.Yu. Tretyakov et al, J. Mol. Spect. v.231, pp.1-14 (2005).
c     B.J. Drouin, JQSRT v.105, pp.450-458 (2007).
c     D.S. Makarov et al, J. Mol. Spect. v.252, pp.242-243 (2008).
c     M.A. Koshelev et al, JQSRT, v.154, pp.24-27 (2015).
c     M.A. Koshelev et al, JQSRT, v.169, pp.91-95 (2016).
c     D.S. Makarov et al, JQSRT v.243 Mar. 2020, doi:10.1016/j.jqsrt.2019.106798
c     Non-resonant intensity from JPL catalog.
c     notes:
c     1. The mm line-width and mixing coefficients are from Tretyakov et al;
c        submm line-widths from Golubiatnikov & Krupnov (except 
c        234 GHz from Drouin)
c     2. The same temperature dependence (X) is used for submillimeter 
c        line widths as in the 60 GHz band: (1/T)**X 
C     3. subroutine ZEEMAN2 is called for Zeeman splitting.
c
c     Local variables:
      INTEGER K,NL
      PARAMETER (NL=49)
      REAL TH,TH1,B,DEN,DFNR,SUM,STR,Y,SF1,SF2,FCEN,DEL,PE1
      REAL DF,X,WB300,W300(NL),F(NL),Y300(NL),S300(NL),Y1(NL),BE(NL)
      COMPLEX CFAC
C      LINES ARE ARRANGED 1-,1+,...37-,37+ IN SPIN-ROTATION SPECTRUM;
c      BY FREQUENCY IN SUBMM SPECTRUM.
C     F,S300,BE from JPL catalog
      DATA F/118.7503, 56.2648, 62.4863, 58.4466, 60.3061, 59.5910,
     & 59.1642, 60.4348, 58.3239, 61.1506, 57.6125, 61.8002,
     & 56.9682, 62.4112, 56.3634, 62.9980, 55.7838, 63.5685,
     & 55.2214, 64.1278, 54.6712, 64.6789, 54.1300, 65.2241,
     & 53.5958, 65.7648, 53.0669, 66.3021, 52.5424, 66.8368,
     & 52.0214, 67.3696, 51.5033, 67.9008, 50.9877, 68.4310,
     & 50.4742, 68.9603, 
     & 233.9461, 368.4982, 401.7398, 424.7630,
     & 487.2493, 566.8956, 715.3929, 731.1867,
     & 773.8395, 834.1456, 895.0711/
      DATA S300/ 0.2925E-14, 0.8011E-15, 0.2460E-14, 0.2208E-14,
     & 0.3323E-14, 0.3265E-14, 0.3690E-14, 0.3859E-14,
     & 0.3610E-14, 0.3974E-14, 0.3200E-14, 0.3686E-14,
     & 0.2607E-14, 0.3131E-14, 0.1967E-14, 0.2459E-14,
     & 0.1382E-14, 0.1796E-14, 0.9074E-15, 0.1225E-14,
     & 0.5581E-15, 0.7818E-15, 0.3222E-15, 0.4684E-15,
     & 0.1749E-15, 0.2636E-15, 0.8937E-16, 0.1397E-15,
     & 0.4301E-16, 0.6968E-16, 0.1952E-16, 0.3277E-16,
     & 0.8354E-17, 0.1454E-16, 0.3377E-17, 0.6091E-17,
     & 0.1289E-17, 0.2409E-17,
     & 0.8298E-17, 0.6506E-15, 0.1347E-16, 0.7097E-14,
     & 0.3032E-14, 0.1815E-16, 0.1839E-14, 0.2215E-16,
     & 0.1160E-13, 0.3999E-14, 0.2537E-16/
      DATA BE/  0.0095,  0.0145,  0.0829,  0.0833,  0.2072,  0.2073,
     &  0.3865,  0.3864,  0.6209,  0.6207,  0.9103,  0.9100,
     &  1.2548,  1.2543,  1.6542,  1.6536,  2.1084,  2.1078,
     &  2.6175,  2.6168,  3.1814,  3.1806,  3.7998,  3.7989,
     &  4.4729,  4.4719,  5.2003,  5.1992,  5.9821,  5.9809,
     &  6.8180,  6.8168,  7.7080,  7.7067,  8.6520,  8.6506,
     &  9.6496,  9.6481,
     &  0.0187, 0.0485, 0.0448, 0.0440,
     &  0.0490, 0.0838, 0.1452, 0.1359,  0.1405, 0.1453, 0.2010/

C      WIDTHS IN MHZ/MB (1- width = gamma_0)
      DATA WB300/.56/, X/.754/
      DATA W300/1.708, 1.703, 1.513, 1.495, 1.433, 1.408,
     & 1.353, 1.353, 1.303, 1.319, 1.262, 1.265, 
     & 1.238, 1.217, 1.207, 1.207, 1.137, 1.137,
     & 1.101, 1.101, 1.037, 1.038, 0.996, 0.996,
     & 2*0.955, 2*0.906, 2*0.858, 
     & 2*0.811, 2*0.764, 2*0.717, 
     & 2*0.669, 1.65, 3*1.64, 4*1.60, 1.62, 2*1.47/
C
c 1st-order mixing coeff. in 1/bar (from o2_abs.f)
      DATA Y300/ -0.041, 0.277, -0.372, 0.559, -0.573, 0.618,
     & -0.366, 0.278, -0.089, -0.021, 0.060, -0.152,
     & 0.216, -0.293, 0.373, -0.436, 0.491, -0.542,
     & 0.571, -0.613, 0.636, -0.670, 0.690, -0.718,
     & 0.740, -0.763, 0.788, -0.807, 0.834, -0.849,
     & 0.876, -0.887, 0.915, -0.922, 0.950, -0.955,
     & 0.987, -0.988, 11*0./
      DATA Y1/ 0., 0.124, -0.002, 0.008, 0.045, -0.093,
     & 0.264, -0.351, 0.359, -0.416, 0.326, -0.353,
     & 0.484, -0.503, 0.579, -0.590, 0.616, -0.619,
     & 0.611, -0.609, 0.574, -0.568, 0.574, -0.566,
     & 0.60, -0.59, 0.63, -0.62, 0.64, -0.63, 0.65,
     & -0.64, 0.65, -0.64, 0.65, -0.64, 0.64, -0.62, 11*0./
C
      GPI = (0.,0.)
      GP = GPI
      GM = GPI
      TH = 300./TEMP
      TH1 = TH-1.
      B = TH**X
      DEN = .001*PRESDA*B
      DFNR = WB300*DEN
      PE1 = .99*DEN
c  1.571e-17 (o16-o16) + 1.3e-19 (o16-o18) = 1.584e-17
      SUM = 1.584E-17*FREQ*FREQ*DFNR/(TH*(FREQ*FREQ + DFNR*DFNR))
      DO 32 K=1,NL
      DF = W300(K)*DEN
      FCEN = F(K)
      DEL = FREQ-FCEN
      Y = PE1*(Y300(K)+Y1(K)*TH1)
      STR = S300(K)*EXP(-BE(K)*TH1)*(FREQ/FCEN)**2
      IF(K.EQ.LINE .AND. K.LE.38) THEN
C  current version of zeeman2 is valid only for spin-rotation lines,
c  for which the index number is equal to the larger J.
        SF1 = 0.
        CALL ZEEMAN2(LINE,DF,TEMP,F(K),DEL,BFIELD,GM,GPI,GP)
c .5*.20946e-4/(1.38065e-19*300)=2.52851e11 (.5 for amplitude instead of power)
        CFAC = CMPLX(1.,Y)*(2.52851e11*STR*PRESDA*TH**3)
        GM = GM*CFAC
        GPI = GPI*CFAC
        GP = GP*CFAC
      ELSE
        SF1 = (DF + DEL*Y)/(DEL**2 + DF**2)
      ENDIF
      SF2 = (DF - (FREQ+FCEN)*Y)/((FREQ+FCEN)**2 + DF**2)
      SUM = SUM + STR*(SF1+SF2)
32    CONTINUE
c   .20946e-4/(3.14159*1.38065e-19*300) = 1.6097e11
      ALPHA = 1.6097E11*SUM*PRESDA*TH**3
      ALPHA = AMAX1(ALPHA,0.)
      RETURN
      END
