      SUBROUTINE TBARRAY2(NLEV, H, TEMP, PRES, H2OVAPOR, H2OLIQUID,
     & OZONE, NANG, SECANT1, SECANT2, NFREQ, FREQ, TB1, TB2, E1, E2,
     & OPACITY)
C
C  COMPUTES MICROWAVE EMISSION AND TRANSMISSION FOR AN ATMOSPHERIC 
C  PROFILE FOR TWO PATHS, AT MULTIPLE ANGLES FOR EACH PATH: 
C  PATH 1: PROPAGATION FROM LEVEL NLEV TO LEVEL 1
C  PATH 2: PROPAGATION FROM LEVEL 1 TO LEVEL NLEV
C
C  REFLECTION AT BOUNDARIES AND THE COSMIC BACKGROUND ARE TO BE
C  CONSIDERED IN THE CALLING PROGRAM, E.G. FOR AN OBSERVER AT LEVEL 1
C  WITH SURFACE AT LEVEL NLEV,
C  TB = TB1 + E1*( (1.-Refl)*Tsurf + Refl*(TB2 + E2*TBcosmic) );
C  OR, FOR AN OBSERVER AT LEVEL NLEV WITH SURFACE AT LEVEL 1,
C  TB = TB2 + E2*( (1.-Refl)*Tsurf + Refl*(TB1 + E1*TBcosmic) );
c
C   PWR - Aug 1, 2000
c         Nov. 3, 2000 removed restriction on transmittance range
C         Aug. 28, 2003 Lambertian option
C         July 9, 2007 corrected bug in liquid calculation
c         Feb. 14, 2018 this version uses height as the vertical coordinate
C         Feb. 5, 2020 calc pda for absn2
c         Apr. 24, 2023 multiple freq, added argument opacity
C
      IMPLICIT NONE
C  ARGUMENTS
c  Note: H2OLIQUID and OZONE can be set to zeros if not needed.
C        For specular reflection at the surface, SECANT1 and
c        SECANT2 would be identical (both positive).
c        For Lambertian scattering at the surface, set the 
c        secant for the downward-propagating path to any value <1.
c
C   INPUTS
      INTEGER NLEV        !no. of atmospheric levels
      REAL H(NLEV)        !height (km)
      REAL TEMP(NLEV)     !temperature (K)
      REAL PRES(NLEV)     !pressure (hPa)
      REAL H2OVAPOR(NLEV) !H2O vapor density (g/m**3)
      REAL H2OLIQUID(NLEV)!H2O liquid density (g/m**3)
      REAL OZONE(NLEV)    !O3 number density (molecules/m**3)
      INTEGER NANG        !no. of angles
      REAL SECANT1(NANG)  !secant of propagation angle (measured from 
c                          vertical) along path 1
      REAL SECANT2(NANG)  !secant of propagation angle along path 2
      INTEGER NFREQ       !no. of frequencies
      REAL FREQ(NFREQ)    !frequencies (GHz)
C   OUTPUTS
      REAL TB1(NFREQ,NANG)!brightness temperature (K) for path 1; e.g.,
c                          emerging from the atmosphere at level 1
      REAL TB2(NFREQ,NANG)!brightness temperature (K) for path 2; e.g.,
c                          emerging from the atmosphere at level NLEV
      REAL E1(NFREQ,NANG) !total transmittance for path 1
      REAL E2(NFREQ,NANG) !total transmittance for path 2
      REAL OPACITY(NFREQ,NLEV) !opacities of each layer (I-1 to I)
c
c---------------------------------------------------------------------
c   LOCAL VARIABLES
      INTEGER I,J,K
      REAL EM,TAV,PAV,PDA,WVAV,WLAV,O3AV,ABSCOEF,TRAN_SLAB
      REAL ABLIQ,ABSN2,ABSO3,TOTOP1,SECL,SEC1,SEC2
C
      DO I=2,NLEV !first I loop
C  use the 'absorption-of-averages' method to compute optical
c  depth of each slab; see M.J. Schwartz, Ph.D. thesis pp. 84-87.
        TAV = (TEMP(I) + TEMP(I-1))/2.
        PAV = SQRT(PRES(I)*PRES(I-1))
        WVAV = (H2OVAPOR(I) + H2OVAPOR(I-1))/2.
        WLAV = (H2OLIQUID(I) + H2OLIQUID(I-1))/2.
        O3AV = (OZONE(I) + OZONE(I-1))/2. 
        PDA = PAV -WVAV*TAV/216.68 !dry-air partial pressure
        CALL O2_ABS(TAV,PAV,WVAV,NFREQ,FREQ,E1(1,1)) ! temporary storage in E1
        CALL H2OABS(TAV,PAV,WVAV,NFREQ,FREQ,E2(1,1)) ! temporary storage in E2
        DO K=1,NFREQ !first K loop
          ABSCOEF = E1(K,1) + E2(K,1) + ABSN2(TAV,PDA,FREQ(K)) +
     &      ABLIQ(WLAV,FREQ(K),TAV) + ABSO3(TAV,PAV,O3AV,FREQ(K))
          OPACITY(K,I) = ABSCOEF*ABS(H(I)-H(I-1))
        END DO !first K loop
      END DO !first I loop
C
      DO K=1,NFREQ !second K loop
        OPACITY(K,1) = 0.
C  approximate a Lambertian distribution by a single angle
c  (ref: eq.7 of P.W. Rosenkranz and C.D. Barnet, JGR v.111, D09S07, 2006.)
        TOTOP1 = 0.
        DO I=2,NLEV !second I loop
          TOTOP1 = TOTOP1 + OPACITY(K,I)
        END DO ! second I loop
        SECL = 1.55 - .16*ALOG(TOTOP1 + .06)
        SECL = AMAX1(SECL, 1.)
        DO J=1,NANG
C  initialize at each angle
         TB1(K,J) = 0.
         TB2(K,J) = 0.
         E1(K,J) = 1.
         E2(K,J) = 1.
         SEC1 = SECANT1(J)
         IF(SEC1.LT.1.) SEC1 = SECL
         SEC2 = SECANT2(J)
         IF(SEC2.LT.1.) SEC2 = SECL
         DO I=2,NLEV !third I loop
           TAV = (TEMP(I) + TEMP(I-1))/2.
C   trace path 1 using integral form of RTE
           EM = E1(K,J)
           E1(K,J) = E1(K,J)*EXP(-SEC1*OPACITY(K,I))
           TB1(K,J) = TB1(K,J) + TAV*(EM-E1(K,J))
C   trace path 2 using differential form of RTE
           TRAN_SLAB = EXP(-SEC2*OPACITY(K,I))
           TB2(K,J) = TAV + TRAN_SLAB*(TB2(K,J)-TAV)
           E2(K,J) = E2(K,J)*TRAN_SLAB
         END DO !third I loop
        END DO !J loop
      END DO !second K loop
      RETURN
      END

