      SUBROUTINE ZEEMAN2(JU,DFC,T,FL,DEL,BFIELD,RM,RPI,RP)
C
C   LANGUAGE - FORTRAN 77
C   PROGRAMMER- P. ROSENKRANZ
c revisions
c 1/4/19 use g coeff from Larsson et al.
c 1/14/19 change argument list; chg. name to zeeman2
c 1/28/23 update for Fortran2018 (no comput. change)
c 7/29/23 max. JU=38

C
C   PURPOSE-
C   COMPUTE INTENSITIES, RELATIVE TO THE UNSPLIT LINE, FOR ONE ZEEMAN-SPLIT
C   OXYGEN LINE IN THE SPIN-ROTATION BAND, SPECIFIED BY JU.
C   COMPLEX ERROR FUNCTION IS USED FOR LINE SHAPE.
C
      IMPLICIT NONE
C  ARGUMENT SPECIFICATIONS-
      INTEGER JU
      REAL T,DEL,BFIELD,DFC,FL
      COMPLEX RM,RPI,RP
C
C  NAME    IN/OUT  UNITS    DESCRIPTION
C  JU        I              LARGER OF THE TWO ANGULAR MOMENTUM QUANTUM 
C                           NUMBERS:  N+1 FOR + BRANCH, N FOR - BRANCH.
C  DFC       I     GHZ      PRESSURE-BROADENING PARAMETER (HWHM)
C  T         I     KELVIN   ATMOSPHERIC TEMPERATURE
C  FL        I     GHZ      LINE CENTER FREQUENCY
C  DEL       I     GHZ      FREQUENCY OFFSET FROM LINE CENTER
C  BFIELD    I     GAUSS    MAGNETIC FIELD MAGNITUDE
C  RM        O              RELATIVE INTENSITY FOR DELTA M = -1
C  RPI       O              RELATIVE INTENSITY FOR DELTA M =  0
C  RP        O              RELATIVE INTENSITY FOR DELTA M = +1
C*******************************************************************
C     REFERENCES FOR EQUATIONS AND COEFFICIENTS:
C     P.W. Rosenkranz, CHAP. 2 in ATMOSPHERIC REMOTE SENSING
C       BY MICROWAVE RADIOMETRY (M.A. Janssen, ed., 1993) 
C       (http://hdl.handle.net/1721.1/68611).
C     R. Larsson et al., JQSRT V.224, pp. 431-438 (2019).

C  LOCAL VARIABLES
      INTEGER JL,M,MN,MN1,MP,N,N1
      real gjnp1(37),gjn(37),gjnm1(37)
      REAL S0,SM,SP,ALPHA,BB,AN,AM,ANN,DN,F0,FM,FP
      DOUBLE COMPLEX DCERROR,G0,GM1,GP1
      DOUBLE PRECISION X,Y,FAC
      data gjnp1/0.970122,0.656846,0.495924,0.398126,0.33246,0.285348,
     & 0.24991,0.222292,0.200164,0.18204,0.166924,0.154125,
     & 0.143148,0.133631,0.125301,0.117948,0.111411,0.10556,
     & 0.100294,0.095528,0.091196,0.087239,0.083612,0.080274,
     & 0.077194,0.074341,0.071691,0.069224,0.066922,0.064768,
     & 0.062749,0.060852,0.059066,0.057382,0.055792,0.054288,0.052863/
      data gjn/1.0011,0.333777,0.166947,0.100214,0.066848,0.047782,
     & 0.035865,0.027921,0.02236,0.018316,0.015282,0.012949,
     & 0.011116,0.009649,0.008458,0.007476,0.006658,0.00597,
     & 0.005384,0.004883,0.004449,0.004072,0.003743,0.003453,
     & 0.003196,0.002968,0.002764,0.002582,0.002417,0.002269,
     & 0.002134,0.002012,0.0019,0.001798,0.001705,0.001619,0.00154/
      data gjnm1/0.,-0.838219,-0.636229,-0.489783,-0.395594,-0.331162,
     & -0.284562,-0.249366,-0.221873,-0.199816,-0.181733,-0.166642,
     & -0.153859,-0.142893,-0.133383,-0.125058,-0.117708,-0.111174,
     & -0.105325,-0.10006,-0.095295,-0.090963,-0.087007,-0.08338,
     & -0.080043,-0.076962,-0.07411,-0.07146,-0.068994,-0.066691,
     & -0.064537,-0.062518,-0.060621,-0.058835,-0.057152,-0.055561,
     & -0.054057/
C
      IF(JU.GT.38 .OR. JU.LT.1 .OR. FL.GT.119.) THEN
        WRITE(*,*) 'Zeeman argument JU=',JU,' is invalid.'
        STOP
      ENDIF
      BB = 1.39984e-3*BFIELD
      ALPHA = .760E-7*FL*SQRT(T)
      Y = DFC/ALPHA
      JL = JU-1
      N = (JL/2)*2+1
      AN = N
      ANN = N*(N+1)
      G0 = DCMPLX(0.D0,0.D0)
      GP1 = G0
      GM1 = G0
      IF(JL.EQ.N) GOTO 20
C
C     LINE IN N- BRANCH
      DN = AN*(2.*AN-1.)*(2.*AN+1.)
      N1 = N-1
      MN1 = -N1
      DO MP=MN1,N1
C   MP is in the lower level (smaller J)
      AM = MP
      F0 = BB*(GJN(N)-GJNM1(N))*AM
      FP = BB*(GJN(N)*(AM-1.)-GJNM1(N)*AM)
      FM = BB*(GJN(N)*(AM+1.)-GJNM1(N)*AM)
      S0 = 3.*(AN+AM)*(AN-AM)/DN
      X = (F0-DEL)/ALPHA
      G0 = G0 + S0*DCERROR(X,Y)
      AM = MP-1
      SP = .75*(AN-AM)*(AN-AM-1.)/DN
      X = (FP-DEL)/ALPHA
      GP1 = GP1 + SP*DCERROR(X,Y)
      AM = MP+1
      SM = .75*(AN+AM)*(AN+AM-1.)/DN
      X = (FM-DEL)/ALPHA
      GM1 = GM1 + SM*DCERROR(X,Y)
      END DO
      GO TO 40
C
C     LINE IN N+ BRANCH
20    CONTINUE
      DN = (AN+1.)*(2.*AN+1.)*(2.*AN+3.)
      MN = -N
      DO M=MN,N
c  M is in the upper level (smaller J)
      AM = M
      F0 = BB*(GJN(N)-GJNP1(N))*AM
      FP = BB*(GJN(N)*AM-GJNP1(N)*(AM+1.))
      FM = BB*(GJN(N)*AM-GJNP1(N)*(AM-1.))
      S0 = 3.*(AN+1.+AM)*(AN+1.-AM)/DN
      X = (F0-DEL)/ALPHA
      G0 = G0 + S0*DCERROR(X,Y)
      SP = .75*(AN+AM+1.)*(AN+AM+2.)/DN
      X = (FP-DEL)/ALPHA
      GP1 = GP1 + SP*DCERROR(X,Y)
      SM = .75*(AN-AM+1.)*(AN-AM+2.)/DN
      X = (FM-DEL)/ALPHA
      GM1 = GM1 + SM*DCERROR(X,Y)
      END DO
40    CONTINUE
      FAC = 1.7724539D0*ALPHA !common factor
      RPI = G0/FAC
      RP = GP1/FAC
      RM = GM1/FAC
      RETURN
      END

